@@ -28,11 +28,12 @@
 import os
 import sys
 import logging
+import shlex
 import bb
 import bb.msg
 import bb.process
 from contextlib import nested
-from bb import data, event, mkdirhier, utils
+from bb import data, event, utils
 
 bblogger = logging.getLogger('BitBake')
 logger = logging.getLogger('BitBake.Build')
@@ -52,16 +53,16 @@ class FuncFailed(Exception):
         self.logfile = logfile
         self.name = name
         if name:
-            self.message = "Function '%s' failed" % name
+            self.msg = "Function '%s' failed" % name
         else:
-            self.message = "Function failed"
+            self.msg = "Function failed"
 
     def __str__(self):
         if self.logfile and os.path.exists(self.logfile):
             msg = ("%s (see %s for further information)" %
-                   (self.message, self.logfile))
+                   (self.msg, self.logfile))
         else:
-            msg = self.message
+            msg = self.msg
         return msg
 
 class TaskBase(event.Event):
@@ -94,13 +95,11 @@ class TaskFailed(TaskBase):
         self.logfile = logfile
         super(TaskFailed, self).__init__(task, metadata)
 
-class InvalidTask(Exception):
-    def __init__(self, task, metadata):
-        self.task = task
-        self.metadata = metadata
+class TaskInvalid(TaskBase):
 
-    def __str__(self):
-        return "No such task '%s'" % self.task
+    def __init__(self, task, metadata):
+        super(TaskInvalid, self).__init__(task, metadata)
+        self._message = "No such task '%s'" % task
 
 
 class LogTee(object):
@@ -124,7 +123,7 @@ class LogTee(object):
         return '<LogTee {0}>'.format(self.name)
 
 
-def exec_func(func, d, dirs = None, logfile = NULL):
+def exec_func(func, d, dirs = None):
     """Execute an BB 'function'"""
 
     body = data.getVar(func, d)
@@ -154,7 +153,6 @@ def exec_func(func, d, dirs = None, logfile = NULL):
             adir = None
 
     ispython = flags.get('python')
-    fakeroot = flags.get('fakeroot')
 
     lockflag = flags.get('lockfiles')
     if lockflag:
@@ -163,13 +161,14 @@ def exec_func(func, d, dirs = None, logfile = NULL):
         lockfiles = None
 
     tempdir = data.getVar('T', d, 1)
+    bb.utils.mkdirhier(tempdir)
     runfile = os.path.join(tempdir, 'run.{0}.{1}'.format(func, os.getpid()))
 
     with bb.utils.fileslocked(lockfiles):
         if ispython:
-            exec_func_python(func, d, runfile, logfile, cwd=adir)
+            exec_func_python(func, d, runfile, cwd=adir)
         else:
-            exec_func_shell(func, d, runfile, logfile, cwd=adir, fakeroot=fakeroot)
+            exec_func_shell(func, d, runfile, cwd=adir)
 
 _functionfmt = """
 def {function}(d):
@@ -178,23 +177,22 @@ def {function}(d):
 {function}(d)
 """
 logformatter = bb.msg.BBLogFormatter("%(levelname)s: %(message)s")
-def exec_func_python(func, d, runfile, logfile, cwd=None):
+def exec_func_python(func, d, runfile, cwd=None):
     """Execute a python BB 'function'"""
 
     bbfile = d.getVar('FILE', True)
-    olddir = os.getcwd()
     code = _functionfmt.format(function=func, body=d.getVar(func, True))
     bb.utils.mkdirhier(os.path.dirname(runfile))
     with open(runfile, 'w') as script:
         script.write(code)
 
     if cwd:
+        try:
+            olddir = os.getcwd()
+        except OSError:
+            olddir = None
         os.chdir(cwd)
 
-    handler = logging.StreamHandler(logfile)
-    handler.setFormatter(logformatter)
-    bblogger.addHandler(handler)
-
     try:
         comp = utils.better_compile(code, func, bbfile)
         utils.better_exec(comp, {"d": d}, code, bbfile)
@@ -204,10 +202,13 @@ def exec_func_python(func, d, runfile, logfile, cwd=None):
 
         raise FuncFailed(func, None)
     finally:
-        bblogger.removeHandler(handler)
-        os.chdir(olddir)
+        if cwd and olddir:
+            try:
+                os.chdir(olddir)
+            except OSError:
+                pass
 
-def exec_func_shell(function, d, runfile, logfile, cwd=None, fakeroot=False):
+def exec_func_shell(function, d, runfile, cwd=None):
     """Execute a shell function from the metadata
 
     Note on directory behavior.  The 'dirs' varflag should contain a list
@@ -223,21 +224,28 @@ def exec_func_shell(function, d, runfile, logfile, cwd=None, fakeroot=False):
         data.emit_func(function, script, d)
 
         script.write("set -x\n")
+        if cwd:
+            script.write("cd %s\n" % cwd)
         script.write("%s\n" % function)
-        os.fchmod(script.fileno(), 0775)
 
-    if fakeroot:
-        cmd = ['fakeroot', runfile]
+    os.chmod(runfile, 0775)
+
+    cmd = runfile
+    if d.getVarFlag(function, 'fakeroot'):
+        fakerootcmd = d.getVar('FAKEROOT', True)
+        if fakerootcmd:
+            cmd = [fakerootcmd, runfile]
+
+    if bb.msg.loggerDefaultVerbose:
+        logfile = LogTee(logger, sys.stdout)
     else:
-        cmd = runfile
-
-    if logger.isEnabledFor(logging.DEBUG):
-        logfile = LogTee(logger, logfile)
+        logfile = sys.stdout
 
     try:
-        bb.process.run(cmd, cwd=cwd, shell=False, stdin=NULL, log=logfile)
+        bb.process.run(cmd, shell=False, stdin=NULL, log=logfile)
     except bb.process.CmdError:
-        raise FuncFailed(function, logfile.name)
+        logfn = d.getVar('BB_LOGFILE', True)
+        raise FuncFailed(function, logfn)
 
 def _task_data(fn, task, d):
     localdata = data.createCopy(d)
@@ -249,14 +257,16 @@ def _task_data(fn, task, d):
     data.expandKeys(localdata)
     return localdata
 
-def exec_task(fn, task, d):
+def _exec_task(fn, task, d, quieterr):
     """Execute a BB 'task'
 
     Execution of a task involves a bit more setup than executing a function,
     running it with its own local metadata, and with some useful variables set.
     """
     if not data.getVarFlag(task, 'task', d):
-        raise InvalidTask(task, d)
+        event.fire(TaskInvalid(task, d), d)
+        logger.error("No such task: %s" % task)
+        return 1
 
     logger.debug(1, "Executing task %s", task)
 
@@ -279,18 +289,62 @@ def exec_task(fn, task, d):
     prefuncs = localdata.getVarFlag(task, 'prefuncs', expand=True)
     postfuncs = localdata.getVarFlag(task, 'postfuncs', expand=True)
 
-    logfile = open(logfn, 'w')
+    # Handle logfiles
+    si = file('/dev/null', 'r')
+    try:
+        logfile = file(logfn, 'w')
+    except OSError:
+        logger.exception("Opening log file '%s'", logfn)
+        pass
+
+    # Dup the existing fds so we dont lose them
+    osi = [os.dup(sys.stdin.fileno()), sys.stdin.fileno()]
+    oso = [os.dup(sys.stdout.fileno()), sys.stdout.fileno()]
+    ose = [os.dup(sys.stderr.fileno()), sys.stderr.fileno()]
+
+    # Replace those fds with our own
+    os.dup2(si.fileno(), osi[1])
+    os.dup2(logfile.fileno(), oso[1])
+    os.dup2(logfile.fileno(), ose[1])
+
+    # Ensure python logging goes to the logfile
+    handler = logging.StreamHandler(logfile)
+    handler.setFormatter(logformatter)
+    # Always enable full debug output into task logfiles
+    handler.setLevel(logging.DEBUG - 2)
+    bblogger.addHandler(handler)
+
+    localdata.setVar('BB_LOGFILE', logfn)
+
     event.fire(TaskStarted(task, localdata), localdata)
     try:
         for func in (prefuncs or '').split():
-            exec_func(func, localdata, logfile=logfile)
-        exec_func(task, localdata, logfile=logfile)
+            exec_func(func, localdata)
+        exec_func(task, localdata)
         for func in (postfuncs or '').split():
-            exec_func(func, localdata, logfile=logfile)
+            exec_func(func, localdata)
     except FuncFailed as exc:
-        event.fire(TaskFailed(exc.name, exc.logfile, localdata), localdata)
-        raise
+        if not quieterr:
+            logger.error(str(exc))
+            event.fire(TaskFailed(task, logfn, localdata), localdata)
+        return 1
     finally:
+        sys.stdout.flush()
+        sys.stderr.flush()
+
+        bblogger.removeHandler(handler)
+
+        # Restore the backup fds
+        os.dup2(osi[0], osi[1])
+        os.dup2(oso[0], oso[1])
+        os.dup2(ose[0], ose[1])
+
+        # Close the backup fds
+        os.close(osi[0])
+        os.close(oso[0])
+        os.close(ose[0])
+        si.close()
+
         logfile.close()
         if os.path.exists(logfn) and os.path.getsize(logfn) == 0:
             logger.debug(2, "Zero size logfn %s, removing", logfn)
@@ -301,6 +355,24 @@ def exec_task(fn, task, d):
     if not localdata.getVarFlag(task, 'nostamp') and not localdata.getVarFlag(task, 'selfstamp'):
         make_stamp(task, localdata)
 
+    return 0
+
+def exec_task(fn, task, d):
+    try: 
+        quieterr = False
+        if d.getVarFlag(task, "quieterrors") is not None:
+            quieterr = True
+
+        return _exec_task(fn, task, d, quieterr)
+    except Exception:
+        from traceback import format_exc
+        if not quieterr:
+            logger.error("Build of %s failed" % (task))
+            logger.error(format_exc())
+            failedevent = TaskFailed(task, None, d)
+            event.fire(failedevent, d)
+        return 1
+
 def stamp_internal(taskname, d, file_name):
     """
     Internal stamp helper function
@@ -310,16 +382,22 @@ def stamp_internal(taskname, d, file_name):
     In the bitbake core, d can be a CacheData and file_name will be set.
     When called in task context, d will be a data store, file_name will not be set
     """
+    taskflagname = taskname
+    if taskname.endswith("_setscene") and taskname != "do_setscene":
+        taskflagname = taskname.replace("_setscene", "")
+
     if file_name:
-        stamp = d.stamp[file_name]
+        stamp = d.stamp_base[file_name].get(taskflagname) or d.stamp[file_name]
+        extrainfo = d.stamp_extrainfo[file_name].get(taskflagname) or ""
     else:
-        stamp = d.getVar('STAMP', True)
+        stamp = d.getVarFlag(taskflagname, 'stamp-base', True) or d.getVar('STAMP', True)
         file_name = d.getVar('BB_FILENAME', True)
+        extrainfo = d.getVarFlag(taskflagname, 'stamp-extra-info', True) or ""
 
     if not stamp:
         return
 
-    stamp = bb.parse.siggen.stampfile(stamp, file_name, taskname)
+    stamp = bb.parse.siggen.stampfile(stamp, file_name, taskname, extrainfo)
 
     bb.utils.mkdirhier(os.path.dirname(stamp))
 
@@ -338,6 +416,12 @@ def make_stamp(task, d, file_name = None):
         f = open(stamp, "w")
         f.close()
 
+    # If we're in task context, write out a signature file for each task
+    # as it completes
+    if not task.endswith("_setscene") and task != "do_setscene" and not file_name:
+        file_name = d.getVar('BB_FILENAME', True)
+        bb.parse.siggen.dump_sigtask(file_name, task, d.getVar('STAMP', True), True)
+
 def del_stamp(task, d, file_name = None):
     """
     Removes a stamp for a given task
@@ -346,8 +430,12 @@ def del_stamp(task, d, file_name = None):
     stamp = stamp_internal(task, d, file_name)
     bb.utils.remove(stamp)
 
-def stampfile(taskname, d):
-    return stamp_internal(taskname, d, None)
+def stampfile(taskname, d, file_name = None):
+    """
+    Return the stamp for a given task
+    (d can be a data dict or dataCache)
+    """
+    return stamp_internal(taskname, d, file_name)
 
 def add_tasks(tasklist, d):
     task_deps = data.getVar('_task_deps', d)
@@ -377,7 +465,9 @@ def add_tasks(tasklist, d):
         getTask('rdeptask')
         getTask('recrdeptask')
         getTask('nostamp')
+        getTask('fakeroot')
         getTask('noexec')
+        getTask('umask')
         task_deps['parents'][task] = []
         for dep in flags['deps']:
             dep = data.expand(dep, d)
