@@ -90,6 +90,17 @@ class DataContext(dict):
         else:
             return value
 
+class ExpansionError(Exception):
+    def __init__(self, varname, expression, exception):
+        self.expression = expression
+        self.variablename = varname
+        self.exception = exception
+        self.msg = "Failure expanding variable %s, expression was %s which triggered exception %s: %s" % (varname, expression, type(exception).__name__, exception)
+        Exception.__init__(self, self.msg)
+        self.args = (varname, expression, exception)
+    def __str__(self):
+        return self.msg
+
 class DataSmart(MutableMapping):
     def __init__(self, special = COWDictBase.copy(), seen = COWDictBase.copy() ):
         self.dict = {}
@@ -117,9 +128,10 @@ class DataSmart(MutableMapping):
                 s = __expand_python_regexp__.sub(varparse.python_sub, s)
                 if s == olds:
                     break
-            except Exception:
-                logger.exception("Error evaluating '%s'", s)
+            except ExpansionError:
                 raise
+            except Exception as exc:
+                raise ExpansionError(varname, s, exc)
 
         varparse.value = s
 
@@ -160,11 +172,12 @@ class DataSmart(MutableMapping):
             if o not in self._seen_overrides:
                 continue
 
-            vars = self._seen_overrides[o]
+            vars = self._seen_overrides[o].copy()
             for var in vars:
                 name = var[:-l]
                 try:
                     self.setVar(name, self.getVar(var, False))
+                    self.delVar(var)
                 except Exception:
                     logger.info("Untracked delVar")
 
@@ -179,11 +192,11 @@ class DataSmart(MutableMapping):
                             keep.append((a ,o))
                             continue
 
-                        if op is "_append":
+                        if op == "_append":
                             sval = self.getVar(append, False) or ""
                             sval += a
                             self.setVar(append, sval)
-                        elif op is "_prepend":
+                        elif op == "_prepend":
                             sval = a + (self.getVar(append, False) or "")
                             self.setVar(append, sval)
 
@@ -246,17 +259,19 @@ class DataSmart(MutableMapping):
         # more cookies for the cookie monster
         if '_' in var:
             override = var[var.rfind('_')+1:]
-            if override not in self._seen_overrides:
-                self._seen_overrides[override] = set()
-            self._seen_overrides[override].add( var )
+            if len(override) > 0:
+                if override not in self._seen_overrides:
+                    self._seen_overrides[override] = set()
+                self._seen_overrides[override].add( var )
 
         # setting var
         self.dict[var]["content"] = value
 
-    def getVar(self, var, exp):
-        value = self.getVarFlag(var, "content")
+    def getVar(self, var, expand=False, noweakdefault=False):
+        value = self.getVarFlag(var, "content", False, noweakdefault)
 
-        if exp and value:
+        # Call expand() separately to make use of the expand cache
+        if expand and value:
             return self.expand(value, var)
         return value
 
@@ -286,18 +301,24 @@ class DataSmart(MutableMapping):
     def delVar(self, var):
         self.expand_cache = {}
         self.dict[var] = {}
+        if '_' in var:
+            override = var[var.rfind('_')+1:]
+            if override and override in self._seen_overrides and var in self._seen_overrides[override]:
+                self._seen_overrides[override].remove(var)
 
     def setVarFlag(self, var, flag, flagvalue):
         if not var in self.dict:
             self._makeShadowCopy(var)
         self.dict[var][flag] = flagvalue
 
-    def getVarFlag(self, var, flag, expand=False):
+    def getVarFlag(self, var, flag, expand=False, noweakdefault=False):
         local_var = self._findVar(var)
         value = None
         if local_var:
             if flag in local_var:
                 value = copy.copy(local_var[flag])
+            elif flag == "content" and "defaultval" in local_var and not noweakdefault:
+                value = copy.copy(local_var["defaultval"])
         if expand and value:
             value = self.expand(value, None)
         return value
@@ -384,18 +405,22 @@ class DataSmart(MutableMapping):
                 yield key
 
     def __iter__(self):
-        seen = set()
-        def _keys(d):
-            if "_data" in d:
-                for key in _keys(d["_data"]):
-                    yield key
-
+        def keylist(d):        
+            klist = set()
             for key in d:
-                if key != "_data":
-                    if not key in seen:
-                        seen.add(key)
-                        yield key
-        return _keys(self.dict)
+                if key == "_data":
+                    continue
+                if not d[key]:
+                    continue
+                klist.add(key)
+
+            if "_data" in d:
+                klist |= keylist(d["_data"])
+
+            return klist
+
+        for k in keylist(self.dict):
+             yield k
 
     def __len__(self):
         return len(frozenset(self))
