@@ -131,7 +131,7 @@ class Git(Fetch):
 
         # If the checkout doesn't exist and the mirror tarball does, extract it
         if not os.path.exists(ud.clonedir) and os.path.exists(repofile):
-            bb.mkdirhier(ud.clonedir)
+            bb.utils.mkdirhier(ud.clonedir)
             os.chdir(ud.clonedir)
             runfetchcmd("tar -xzf %s" % (repofile), d)
 
@@ -188,7 +188,7 @@ class Git(Fetch):
             os.chdir(coprefix)
             runfetchcmd("%s checkout -q -f %s%s" % (ud.basecmd, ud.tag, readpathspec), d)
         else:
-            bb.mkdirhier(codir)
+            bb.utils.mkdirhier(codir)
             os.chdir(ud.clonedir)
             runfetchcmd("%s read-tree %s%s" % (ud.basecmd, ud.tag, readpathspec), d)
             runfetchcmd("%s checkout-index -q -f --prefix=%s -a" % (ud.basecmd, coprefix), d)
@@ -245,18 +245,20 @@ class Git(Fetch):
         revs = bb.persist_data.persist('BB_URI_HEADREVS', d)
 
         key = self.generate_revision_key(url, ud, d, branch=True)
-        rev = revs[key]
-        if rev is None:
+
+        try:
+            return revs[key]
+        except KeyError:
             # Compatibility with old key format, no branch included
             oldkey = self.generate_revision_key(url, ud, d, branch=False)
-            rev = revs[oldkey]
-            if rev is not None:
-                del revs[oldkey]
-            else:
+            try:
+                rev = revs[oldkey]
+            except KeyError:
                 rev = self._latest_revision(url, ud, d)
+            else:
+                del revs[oldkey]
             revs[key] = rev
-
-        return str(rev)
+            return rev
 
     def sortable_revision(self, url, ud, d):
         """
@@ -267,9 +269,9 @@ class Git(Fetch):
         oldkey = self.generate_revision_key(url, ud, d, branch=False)
 
         latest_rev = self._build_revision(url, ud, d)
-        last_rev = localcounts[key + '_rev']
+        last_rev = localcounts.get(key + '_rev')
         if last_rev is None:
-            last_rev = localcounts[oldkey + '_rev']
+            last_rev = localcounts.get(oldkey + '_rev')
             if last_rev is not None:
                 del localcounts[oldkey + '_rev']
                 localcounts[key + '_rev'] = last_rev
@@ -279,9 +281,9 @@ class Git(Fetch):
         if uselocalcount:
             count = Fetch.localcount_internal_helper(ud, d)
         if count is None:
-            count = localcounts[key + '_count']
+            count = localcounts.get(key + '_count')
         if count is None:
-            count = localcounts[oldkey + '_count']
+            count = localcounts.get(oldkey + '_count')
             if count is not None:
                 del localcounts[oldkey + '_count']
                 localcounts[key + '_count'] = count
