@@ -98,7 +98,7 @@ class Osc(Fetch):
             oscfetchcmd = self._buildosccommand(ud, d, "fetch")
             logger.info("Fetch " + loc)
             # check out sources there
-            bb.mkdirhier(ud.pkgdir)
+            bb.utils.mkdirhier(ud.pkgdir)
             os.chdir(ud.pkgdir)
             logger.debug(1, "Running %s", oscfetchcmd)
             runfetchcmd(oscfetchcmd, d)
