@@ -25,26 +25,28 @@ BitBake build tools.
 #
 # Based on functions from the base bb module, Copyright 2003 Holger Schurig
 
-import os, re
+from future_builtins import zip
+import os
+import logging
 import bb
 from   bb import data
 from   bb.fetch import Fetch
 from   bb.fetch import FetchError
-from   bb.fetch import MissingParameterError
+from   bb.fetch import logger
 
 class Perforce(Fetch):
     def supports(self, url, ud, d):
         return ud.type in ['p4']
 
-    def doparse(url,d):
+    def doparse(url, d):
         parm = {}
         path = url.split("://")[1]
         delim = path.find("@");
         if delim != -1:
-            (user,pswd,host,port) = path.split('@')[0].split(":")
+            (user, pswd, host, port) = path.split('@')[0].split(":")
             path = path.split('@')[1]
         else:
-            (host,port) = data.getVar('P4PORT', d).split(':')
+            (host, port) = data.getVar('P4PORT', d).split(':')
             user = ""
             pswd = ""
 
@@ -54,19 +56,19 @@ class Perforce(Fetch):
             plist = path.split(';')
             for item in plist:
                 if item.count('='):
-                    (key,value) = item.split('=')
+                    (key, value) = item.split('=')
                     keys.append(key)
                     values.append(value)
 
-            parm = dict(zip(keys,values))
+            parm = dict(zip(keys, values))
         path = "//" + path.split(';')[0]
         host += ":%s" % (port)
         parm["cset"] = Perforce.getcset(d, path, host, user, pswd, parm)
 
-        return host,path,user,pswd,parm
+        return host, path, user, pswd, parm
     doparse = staticmethod(doparse)
 
-    def getcset(d, depot,host,user,pswd,parm):
+    def getcset(d, depot, host, user, pswd, parm):
         p4opt = ""
         if "cset" in parm:
             return parm["cset"];
@@ -86,19 +88,19 @@ class Perforce(Fetch):
             depot += "@%s" % (p4date)
 
         p4cmd = data.getVar('FETCHCOMMAND_p4', d, 1)
-        bb.msg.debug(1, bb.msg.domain.Fetcher, "Running %s%s changes -m 1 %s" % (p4cmd, p4opt, depot))
+        logger.debug(1, "Running %s%s changes -m 1 %s", p4cmd, p4opt, depot)
         p4file = os.popen("%s%s changes -m 1 %s" % (p4cmd, p4opt, depot))
         cset = p4file.readline().strip()
-        bb.msg.debug(1, bb.msg.domain.Fetcher, "READ %s" % (cset))
+        logger.debug(1, "READ %s", cset)
         if not cset:
             return -1
 
         return cset.split(' ')[1]
     getcset = staticmethod(getcset)
 
-    def localpath(self, url, ud, d):        
+    def localpath(self, url, ud, d):
 
-        (host,path,user,pswd,parm) = Perforce.doparse(url,d)
+        (host, path, user, pswd, parm) = Perforce.doparse(url, d)
 
         # If a label is specified, we use that as our filename
 
@@ -111,12 +113,11 @@ class Perforce(Fetch):
         if which != -1:
             base = path[:which]
 
-        if base[0] == "/":
-            base = base[1:]
+        base = self._strip_leading_slashes(base)
 
         cset = Perforce.getcset(d, path, host, user, pswd, parm)
 
-        ud.localfile = data.expand('%s+%s+%s.tar.gz' % (host,base.replace('/', '.'), cset), d)
+        ud.localfile = data.expand('%s+%s+%s.tar.gz' % (host, base.replace('/', '.'), cset), d)
 
         return os.path.join(data.getVar("DL_DIR", d, 1), ud.localfile)
 
@@ -125,22 +126,14 @@ class Perforce(Fetch):
         Fetch urls
         """
 
-        # try to use the tarball stash
-        if Fetch.try_mirror(d, ud.localfile):
-            bb.msg.debug(1, bb.msg.domain.Fetcher, "%s already exists or was mirrored, skipping perforce checkout." % ud.localpath)
-            return
-
-        (host,depot,user,pswd,parm) = Perforce.doparse(loc, d)
+        (host, depot, user, pswd, parm) = Perforce.doparse(loc, d)
 
         if depot.find('/...') != -1:
             path = depot[:depot.find('/...')]
         else:
             path = depot
 
-        if "module" in parm:
-            module = parm["module"]
-        else:
-            module = os.path.basename(path)
+        module = parm.get('module', os.path.basename(path))
 
         localdata = data.createCopy(d)
         data.setVar('OVERRIDES', "p4:%s" % data.getVar('OVERRIDES', localdata), localdata)
@@ -160,33 +153,33 @@ class Perforce(Fetch):
         p4cmd = data.getVar('FETCHCOMMAND', localdata, 1)
 
         # create temp directory
-        bb.msg.debug(2, bb.msg.domain.Fetcher, "Fetch: creating temporary directory")
-        bb.mkdirhier(data.expand('${WORKDIR}', localdata))
+        logger.debug(2, "Fetch: creating temporary directory")
+        bb.utils.mkdirhier(data.expand('${WORKDIR}', localdata))
         data.setVar('TMPBASE', data.expand('${WORKDIR}/oep4.XXXXXX', localdata), localdata)
         tmppipe = os.popen(data.getVar('MKTEMPDIRCMD', localdata, 1) or "false")
         tmpfile = tmppipe.readline().strip()
         if not tmpfile:
-            bb.error("Fetch: unable to create temporary directory.. make sure 'mktemp' is in the PATH.")
+            logger.error("Fetch: unable to create temporary directory.. make sure 'mktemp' is in the PATH.")
             raise FetchError(module)
 
         if "label" in parm:
-            depot = "%s@%s" % (depot,parm["label"])
+            depot = "%s@%s" % (depot, parm["label"])
         else:
             cset = Perforce.getcset(d, depot, host, user, pswd, parm)
-            depot = "%s@%s" % (depot,cset)
+            depot = "%s@%s" % (depot, cset)
 
         os.chdir(tmpfile)
-        bb.msg.note(1, bb.msg.domain.Fetcher, "Fetch " + loc)
-        bb.msg.note(1, bb.msg.domain.Fetcher, "%s%s files %s" % (p4cmd, p4opt, depot))
+        logger.info("Fetch " + loc)
+        logger.info("%s%s files %s", p4cmd, p4opt, depot)
         p4file = os.popen("%s%s files %s" % (p4cmd, p4opt, depot))
 
         if not p4file:
-            bb.error("Fetch: unable to get the P4 files from %s" % (depot))
+            logger.error("Fetch: unable to get the P4 files from %s", depot)
             raise FetchError(module)
 
         count = 0
 
-        for file in p4file: 
+        for file in p4file:
             list = file.split()
 
             if list[2] == "delete":
@@ -195,11 +188,11 @@ class Perforce(Fetch):
             dest = list[0][len(path)+1:]
             where = dest.find("#")
 
-            os.system("%s%s print -o %s/%s %s" % (p4cmd, p4opt, module,dest[:where],list[0]))
+            os.system("%s%s print -o %s/%s %s" % (p4cmd, p4opt, module, dest[:where], list[0]))
             count = count + 1
-            
+
         if count == 0:
-            bb.error("Fetch:  No files gathered from the P4 fetch")
+            logger.error("Fetch:  No files gathered from the P4 fetch")
             raise FetchError(module)
 
         myret = os.system("tar -czf %s %s" % (ud.localpath, module))
@@ -210,6 +203,4 @@ class Perforce(Fetch):
                 pass
             raise FetchError(module)
         # cleanup
-        os.system('rm -rf %s' % tmpfile)
-
-
+        bb.utils.prunedir(tmpfile)
