@@ -30,12 +30,10 @@ import Queue
 def event_handle_idle_func (eventHandler, build, pbar):
 
     # Consume as many messages as we can in the time available to us
-    try:
-        while 1:
-            event = eventHandler.get(False)
-            build.handle_event (event, pbar)
-    except Queue.Empty:
-        pass
+    event = eventHandler.getEvent()
+    while event:
+        build.handle_event (event, pbar)
+        event = eventHandler.getEvent()
 
     return True
 
@@ -72,6 +70,7 @@ def main (server, eventHandler):
     window = MainWindow ()
     window.show_all ()
     pbar = ProgressBar(window)
+    pbar.connect("delete-event", gtk.main_quit)
 
     # Create the object for the current build
     running_build = RunningBuild ()
@@ -83,8 +82,12 @@ def main (server, eventHandler):
     try:
         cmdline = server.runCommand(["getCmdLineAction"])
         if not cmdline:
+            print("Nothing to do.  Use 'bitbake world' to build everything, or run 'bitbake --help' for usage information.")
             return 1
-        ret = server.runCommand(cmdline)
+        elif not cmdline['action']:
+            print(cmdline['msg'])
+            return 1
+        ret = server.runCommand(cmdline['action'])
         if ret != True:
             print("Couldn't get default commandline! %s" % ret)
             return 1
@@ -106,6 +109,8 @@ def main (server, eventHandler):
         # ignore interrupted io
         if ioerror.args[0] == 4:
             pass
+    except KeyboardInterrupt:
+        pass
     finally:
         server.runCommand(["stateStop"])
 
